@@ -1,6 +1,6 @@
 /*
     gcommon
-    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2013 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -1380,6 +1380,20 @@ GDateTime *g_date_time_ref (GDateTime *datetime);
 GDateTime *g_date_time_new_now_utc (void);
 GDateTime *g_date_time_new_from_unix_utc (gint64 t);
 GDateTime *g_date_time_new_from_timeval_utc (const GTimeVal *tv);
+GDateTime *g_date_time_new_utc (gint year, gint month, gint day, gint hour, gint minute, gdouble seconds);
+GDateTime *g_date_time_add (GDateTime *datetime, GTimeSpan timespan);
+GDateTime *g_date_time_add_years (GDateTime *datetime, gint years);
+GDateTime *g_date_time_add_months (GDateTime *datetime, gint months);
+GDateTime *g_date_time_add_weeks (GDateTime *datetime, gint weeks);
+GDateTime *g_date_time_add_days (GDateTime *datetime, gint days);
+GDateTime *g_date_time_add_hours (GDateTime *datetime, gint hours);
+GDateTime *g_date_time_add_minutes (GDateTime *datetime, gint minutes);
+GDateTime *g_date_time_add_seconds (GDateTime *datetime, gdouble seconds);
+GDateTime *g_date_time_add_full (GDateTime *datetime, gint years, gint months, gint days, gint hours, gint minutes, gdouble seconds);
+gint g_date_time_compare (gconstpointer dt1, gconstpointer dt2);
+GTimeSpan g_date_time_difference (GDateTime *end, GDateTime *begin);
+guint g_date_time_hash (gconstpointer datetime);
+gboolean g_date_time_equal (gconstpointer dt1, gconstpointer dt2);
 void g_date_time_get_ymd (GDateTime *datetime, gint *year, gint *month, gint *day);
 gint g_date_time_get_year (GDateTime *datetime);
 gint g_date_time_get_month (GDateTime *datetime);
@@ -1387,6 +1401,10 @@ gint g_date_time_get_day_of_month (GDateTime *datetime);
 gint g_date_time_get_hour (GDateTime *datetime);
 gint g_date_time_get_minute (GDateTime *datetime);
 gint g_date_time_get_second (GDateTime *datetime);
+gint g_date_time_get_microsecond (GDateTime *datetime);
+gdouble g_date_time_get_seconds (GDateTime *datetime);
+gint64 g_date_time_to_unix (GDateTime *datetime);
+gboolean g_date_time_to_timeval (GDateTime *datetime, GTimeVal *tv);
 #endif /* not GLIB_CHECK_VERSION(2,26,0) */
 
 
@@ -1451,15 +1469,35 @@ typedef enum _GFileTest
 
 gboolean g_file_test (const gchar *file, GFileTest test);
 #endif /* USE_GTK_EMULATE */
+#if ! GLIB_CHECK_VERSION(2,26,0)
+typedef struct stat GStatBuf;
+#endif /* not GLIB_CHECK_VERSION(2,26,0) */
+
+
+#if ! GLIB_CHECK_VERSION(2,26,0)
+gchar *g_mkdtemp (gchar *tmpl);
+gchar *g_mkdtemp_full (gchar *tmpl, gint mode);
+#endif /* not GLIB_CHECK_VERSION(2,26,0) */
+#if ! GLIB_CHECK_VERSION(2,30,0)
+gchar *g_dir_make_tmp (const gchar *tmpl, GError **error);
+#endif /* not GLIB_CHECK_VERSION(2,30,0) */
 #if ! GLIB_CHECK_VERSION(2,22,0)
-# define g_mapped_file_unref g_mapped_file_free
+# define g_mapped_file_unref(file) g_mapped_file_free(file)
 #endif /* not GLIB_CHECK_VERSION(2,22,0) */
 #if ! GLIB_CHECK_VERSION(2,6,0)
 # define g_open open
 # define g_rename rename
-# define g_mkdir mkdir
+# ifdef G_OS_WIN32
+gint g_mkdir (const gchar *filename, gint mode);
+# else /* not G_OS_WIN32 */
+#  define g_mkdir mkdir
+# endif /* not G_OS_WIN32 */
 # define g_stat stat
-# define g_lstat lstat
+# ifdef HAVE_LSTAT
+#  define g_lstat lstat
+# else /* not HAVE_LSTAT */
+#  define g_lstat stat
+# endif /* not HAVE_LSTAT */
 # define g_unlink unlink
 # define g_remove remove
 # define g_rmdir rmdir
@@ -1467,17 +1505,40 @@ gboolean g_file_test (const gchar *file, GFileTest test);
 # define g_freopen freopen
 #endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #if ! GLIB_CHECK_VERSION(2,8,0)
-# define g_chmod chmod
-# define g_access access
-# define g_creat creat
-# define g_chdir chdir
+# ifdef G_OS_WIN32
+gint g_chmod (const gchar *filename, gint mode);
+gint g_access (const gchar *filename, gint mode);
+gint g_creat (const gchar *filename, gint mode);
+gint g_chdir (const gchar *path);
+# else /* not G_OS_WIN32 */
+#  define g_chmod chmod
+#  define g_access access
+#  define g_creat creat
+#  define g_chdir chdir
+# endif /* not G_OS_WIN32 */
 #endif /* not GLIB_CHECK_VERSION(2,8,0) */
 #if ! GLIB_CHECK_VERSION(2,18,0)
-# define g_utime utime
+# ifdef G_OS_WIN32
+# include <sys/utime.h>
+# endif /* G_OS_WIN32 */
+# ifdef HAVE_UTIME_H
+# include <utime.h>
+# endif /* HAVE_UTIME_H */
+gint g_utime (const gchar *filename, struct utimbuf *utb);
 #endif /* not GLIB_CHECK_VERSION(2,18,0) */
-#if ! GLIB_CHECK_VERSION(2,26,0)
-typedef struct stat GStatBuf;
-#endif /* not GLIB_CHECK_VERSION(2,26,0) */
+
+
+/******************************************************************************
+* Directory Utilities                                                         *
+******************************************************************************/
+#ifdef USE_GTK_EMULATE
+typedef struct _GDir GDir;
+
+
+GDir *g_dir_open (const gchar *path, guint flags, GError **error);
+const gchar *g_dir_read_name (GDir *dir);
+void g_dir_close (GDir *dir);
+#endif /* USE_GTK_EMULATE */
 
 
 /******************************************************************************
@@ -1506,19 +1567,6 @@ gchar *g_uri_unescape_segment (const gchar *escaped_string, const gchar *escaped
 #endif /* not GLIB_CHECK_VERSION(2,16,0) */
 
 
-/******************************************************************************
-* Directory Utilities                                                         *
-******************************************************************************/
-#ifdef USE_GTK_EMULATE
-typedef struct _GDir GDir;
-
-
-GDir *g_dir_open (const gchar *path, guint flags, GError **error);
-const gchar *g_dir_read_name (GDir *dir);
-void g_dir_close (GDir *dir);
-#endif /* USE_GTK_EMULATE */
-
-
 /******************************************************************************
 * Windows Compatibility Functions                                             *
 ******************************************************************************/
@@ -1549,25 +1597,63 @@ typedef struct _GList {
   struct _GList *prev;
   struct _GList *next;
 } GList;
+typedef gint (*GCompareDataFunc) (gconstpointer a, gconstpointer b, gpointer user_data);
 typedef gint (*GCompareFunc) (gconstpointer a, gconstpointer b);
+#endif /* USE_GTK_EMULATE */
+#if ! GLIB_CHECK_VERSION(2,4,0)
+typedef gpointer (*GCopyFunc) (gconstpointer src, gpointer data);
+#endif /* not GLIB_CHECK_VERSION(2,4,0) */
+#ifdef USE_GTK_EMULATE
+typedef void (*GDestroyNotify) (gpointer data);
+typedef void (*GFunc) (gpointer data, gpointer user_data);
 
 
 GList *g_list_append (GList *list, gpointer data);
 GList *g_list_prepend (GList *glist, gpointer  data);
+GList *g_list_insert (GList *glist, gpointer data, gint position);
+GList *g_list_insert_before (GList *glist, GList *sibling, gpointer data);
 GList *g_list_insert_sorted (GList *glist, gpointer data, GCompareFunc compare);
 GList *g_list_remove (GList *glist, gconstpointer data);
 GList *g_list_remove_link (GList *glist, GList *glink);
 GList *g_list_delete_link (GList *glist, GList *glink);
+GList *g_list_remove_all (GList *glist, gconstpointer data);
 void g_list_free (GList *glist);
+#endif /* USE_GTK_EMULATE */
+#if ! GLIB_CHECK_VERSION(2,28,0)
+void g_list_free_full (GList *glist, GDestroyNotify destroy);
+#endif /* not GLIB_CHECK_VERSION(2,28,0) */
+#ifdef USE_GTK_EMULATE
+GList *g_list_alloc (void);
+void g_list_free_1 (GList *glist);
+# define g_list_free1 g_list_free_1
 guint g_list_length (GList *glist);
+GList *g_list_copy (GList *glist);
+#endif /* USE_GTK_EMULATE */
+#if ! GLIB_CHECK_VERSION(2,34,0)
+GList *g_list_copy_deep (GList *glist, GCopyFunc func, gpointer user_data);
+#endif /* not GLIB_CHECK_VERSION(2,34,0) */
+#ifdef USE_GTK_EMULATE
+GList *g_list_reverse (GList *glist);
+GList *g_list_sort (GList *glist, GCompareFunc compare_func);
+#endif /* USE_GTK_EMULATE */
+#if ! GLIB_CHECK_VERSION(2,10,0)
+GList *g_list_insert_sorted_with_data (GList *glist, gpointer data, GCompareDataFunc func, gpointer user_data);
+#endif /* not GLIB_CHECK_VERSION(2,10,0) */
+#ifdef USE_GTK_EMULATE
+GList *g_list_sort_with_data (GList *glist, GCompareDataFunc compare_func, gpointer user_data);
 GList *g_list_concat (GList *glist1, GList *glist2);
+void g_list_foreach (GList *glist, GFunc func, gpointer user_data);
 GList *g_list_first (GList *glist);
 GList *g_list_last (GList *glist);
 # define g_list_previous(glist) ((glist)?((GList *)(glist))->prev:NULL)
 # define g_list_next(glist) ((glist)?((GList *)(glist))->next:NULL)
+GList *g_list_nth (GList *glist, guint n);
 gpointer g_list_nth_data (GList *glist, guint n);
+GList *g_list_nth_prev (GList *glist, guint n);
 GList *g_list_find (GList *glist, gconstpointer data);
 GList *g_list_find_custom (GList *glist, gconstpointer data, GCompareFunc compare);
+gint g_list_position (GList *glist, GList *llink);
+gint g_list_index (GList *glist, gconstpointer data);
 #endif /* USE_GTK_EMULATE */
 
 
@@ -1580,7 +1666,6 @@ typedef guint (*GHashFunc) (gconstpointer key);
 typedef gboolean (*GEqualFunc) (gconstpointer a, gconstpointer b);
 typedef void (*GHFunc) (gpointer key, gpointer value, gpointer user_data);
 typedef gboolean (*GHRFunc) (gpointer key, gpointer value, gpointer user_data);
-typedef void (*GDestroyNotify) (gpointer data);
 
 
 GHashTable *g_hash_table_new (GHashFunc hash_func, GEqualFunc key_equal_func);
