@@ -335,6 +335,7 @@ file_open_edit (const gchar *file)
   gchar *filename;
   gint i, page_num;
   TmaidWindow *tmaid;
+  PangoContext *context;
   GdkRectangle rc;
   GtkWidget *table;
 #ifndef G_OS_WIN32
@@ -414,8 +415,13 @@ file_open_edit (const gchar *file)
 #else /* not G_OS_WIN32 */
   preedit = gtk_window_new (GTK_WINDOW_POPUP);
   tmaid->preedit = gtk_drawing_area_new ();
+# if GTK_CHECK_VERSION(3,0,0)
+  g_signal_connect (G_OBJECT (tmaid->preedit), "draw",
+                                    G_CALLBACK (signal_draw_preedit), tmaid);
+# else /* not GTK_CHECK_VERSION(3,0,0) */
   g_signal_connect (G_OBJECT (tmaid->preedit), "expose-event",
                                     G_CALLBACK (signal_expose_preedit), tmaid);
+# endif /* not GTK_CHECK_VERSION(3,0,0) */
   gtk_container_add (GTK_CONTAINER (preedit), tmaid->preedit);
   gtk_widget_show (tmaid->preedit);
 #endif /* not G_OS_WIN32 */
@@ -432,8 +438,13 @@ file_open_edit (const gchar *file)
                                     G_CALLBACK (signal_unrealize), tmaid);
   g_signal_connect (G_OBJECT (tmaid->drawing), "configure-event",
                                     G_CALLBACK (signal_config), tmaid);
+#if GTK_CHECK_VERSION(3,0,0)
+  g_signal_connect (G_OBJECT (tmaid->drawing), "draw",
+                                    G_CALLBACK (signal_draw), tmaid);
+#else /* not GTK_CHECK_VERSION(3,0,0) */
   g_signal_connect (G_OBJECT (tmaid->drawing), "expose-event",
                                     G_CALLBACK (signal_expose), tmaid);
+#endif /* not GTK_CHECK_VERSION(3,0,0) */
   g_signal_connect (G_OBJECT (tmaid->drawing), "button-press-event",
                                     G_CALLBACK (signal_button_press), tmaid);
   g_signal_connect (G_OBJECT (tmaid->drawing), "motion-notify-event",
@@ -454,21 +465,38 @@ file_open_edit (const gchar *file)
                             | GDK_FOCUS_CHANGE_MASK | GDK_SCROLL_MASK);
   gtk_widget_set_can_focus (tmaid->drawing, TRUE);
   /* ja:フォント */
-  if (tmaid->ft.font_name)
+  context = gtk_widget_create_pango_context (window);
+  if (context)
     {
       PangoFontDescription *desc;
 
-      desc = pango_font_description_from_string (tmaid->ft.font_name);
-      gtk_widget_modify_font (window, desc);
-      pango_font_description_free (desc);
+      desc = tmaid->ft.font_name
+            ? pango_font_description_from_string (tmaid->ft.font_name) : NULL;
+      pango_context_set_font_description (context, desc ? desc : system_font);
+      tmaid->layout = pango_layout_new (context);
+      if (desc)
+        pango_font_description_free (desc);
+      g_object_unref (context);
     }
-  else
+  if (!tmaid->layout)
     {
-      gtk_widget_modify_font (window, system_font);
+      if (tmaid->ft.font_name)
+        {
+          PangoFontDescription *desc;
+
+          desc = pango_font_description_from_string (tmaid->ft.font_name);
+          gtk_widget_modify_font (window, desc);
+          pango_font_description_free (desc);
+        }
+      else
+        {
+          gtk_widget_modify_font (window, system_font);
+        }
+      tmaid->layout = gtk_widget_create_pango_layout (window, NULL);
     }
-  tmaid->layout = gtk_widget_create_pango_layout (window, NULL);
   charset_size (tmaid->layout, &tmaid->font_width, &tmaid->font_height,
                                                             tmaid->font_buf);
+
   /* ja:プライマリセレクション */
   gtk_selection_add_targets (tmaid->drawing, GDK_SELECTION_PRIMARY,
                                             targets, G_N_ELEMENTS (targets));
