@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2013 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -96,7 +96,7 @@ main (int   argc,
       char *argv[])
 {
   gboolean *arg_files, success[12];
-  const gchar *file;
+  gchar *file;
   gint i, count = 0, init_line = 0;
   GList *glist;
   GObject *accel, *winmenu;
@@ -125,12 +125,17 @@ main (int   argc,
 {NULL,      '\0', NULL,           NULL,                             0,                   NULL}};
 
   /* ja:初期化 */
+#ifdef G_OS_WIN32
+  SetErrorMode (SetErrorMode (0) | SEM_FAILCRITICALERRORS);
+#endif /* G_OS_WIN32 */
   setlocale (LC_ALL, "");
-#if ! GTK_CHECK_VERSION(3,0,0)
+#if ! GTK_CHECK_VERSION(2,24,0)
   gtk_set_locale ();
-#endif /* not GTK_CHECK_VERSION(3,0,0) */
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   gtk_init (&argc, &argv);
-  bindtextdomain (PACKAGE, misc_spec_locale (argv[0]));
+  file = misc_spec_locale (argv[0]);
+  bindtextdomain (PACKAGE, file);
+  g_free (file);
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 
@@ -197,7 +202,10 @@ TMAID_COPYRIGHT_NOTICE"\n"
   newfile = arg_n.flag;
   /* ja:レジストリ */
   if (!arg_p.flag)
-    file = NULL;
+    {
+      g_free (file);
+      file = NULL;
+    }
   /* ja:ウインドウサイズ */
   def_width = array[0];
   def_height = array[1];
@@ -391,6 +399,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
 
   /* ja:キーファイルに書き込む */
   keyfile_write (file);
+  g_free (file);
 
   pango_font_description_free (system_font);
   g_free (clipboard_text);
